#!/usr/bin/env python
# -*- coding: utf-8 -*-
#*****************************************************************************
#       Copyright (C) 2006 Serge Weinstock <serge.weinstock@ntlworld.com>
#  Distributed under the terms of the BSD License.  The full license is in
#  the file COPYING, distributed as part of this software.
#*****************************************************************************
"""
Miscellaneous utilities
"""
import sys, __builtin__

def property(function):
    """
    A decorator for setting properties

    example:

    class Angle(object):
        def __init__(self,rad):
            self._rad = rad
        @Property
        def rad():
            '''The angle in radians'''
            def fget(self):
                return self._rad
            def fset(self,angle):
                if isinstance(angle,Angle): angle = angle.rad
                self._rad = float(angle)
        @Property
        def deg():
            '''The angle in degrees'''
            def fget(self):
                return degrees(self._rad)
            def fset(self,angle):
                if isinstance(angle,Angle): angle = angle.deg
                self._rad = radians(angle)
    """
    keys = 'fget', 'fset', 'fdel'
    func_locals = {'doc':function.__doc__}
    def probeFunc(frame, event, arg): #@UnusedVariable
        if event == 'return':
            locals = frame.f_locals
            func_locals.update(dict((k,locals.get(k)) for k in keys))
            sys.settrace(None)
        return probeFunc
    sys.settrace(probeFunc)
    function()
    return __builtin__.property(**func_locals)


